import angular from 'angular';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var originalModule = angular.module;

function evaluateFilterFunction(name, def) {
  // call the ngDef function to get the filter function
  // TODO: resolve the dependence by $injector
  return _defineProperty({}, name, def());
}

function _hasDependence($injector, ngDef) {
  var deps = $injector.annotate(ngDef);
  return deps.length > 0;
}

angular.module = function (moduleName) {
  for (var _len = arguments.length, otherArgs = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    otherArgs[_key - 1] = arguments[_key];
  }

  var module = originalModule.apply(void 0, [moduleName].concat(otherArgs));
  var $injector = angular.injector(['ng', moduleName]);

  if (!$injector.has('$ngVue')) {
    throw new Error('ngVue.plugins should be required as a dependency in your application');
  }

  var originalFilter = module.filter;
  var filters = [];
  module.provider('$ngVueFilter', ['$filterProvider', '$ngVueProvider', function ($filterProvider, $ngVueProvider) {
    this.register = function (name, ngDef) {
      $filterProvider.register(name, ngDef);

      if (!_hasDependence($injector, ngDef)) {
        $ngVueProvider.filters.register(evaluateFilterFunction(name, ngDef));
      }
    }; // `$ngVueFilter` only works in the config phase


    this.$get = function () {};
  }]);

  module.filter = function (name, ngDef) {
    if (!_hasDependence($injector, ngDef)) {
      filters.push(evaluateFilterFunction(name, ngDef));
    }

    return originalFilter.apply(this, arguments);
  };

  module.config(['$ngVueProvider', function ($ngVueProvider) {
    filters.forEach(function (f) {
      return $ngVueProvider.filters.register(f);
    });
    filters = [];
  }]);
  return module;
};
